Module: RocketJob::Plugins::ThrottleDependentJobs
- Extended by:
- ActiveSupport::Concern
- Defined in:
- lib/rocket_job/plugins/throttle_dependent_jobs.rb
Overview
Prevent this job from starting, or a batch slice from starting if the dependent jobs are running.
Features:
- Ensures dependent jobs won't run
When the throttle has been exceeded all jobs of this class will be ignored until the
next refresh.
RocketJob::Config::re_check_secondswhich by default is 60 seconds.