Module: RocketJob::Batch::Statistics
- Extended by:
- ActiveSupport::Concern
- Included in:
- Jobs::OnDemandBatchJob
- Defined in:
- lib/rocket_job/batch/statistics.rb
Overview
Allow statistics to be gathered while a batch job is running.
Notes:
- Statistics for successfully processed records within a slice are saved.
- Statistics gathered during a perform that then results in an exception are discarded.
Defined Under Namespace
Classes: Stats
Instance Method Summary collapse
-
#statistics_inc(key, increment = 1) ⇒ Object
Increment a statistic.
Instance Method Details
#statistics_inc(key, increment = 1) ⇒ Object
Increment a statistic
74 75 76 77 78 |
# File 'lib/rocket_job/batch/statistics.rb', line 74 def statistics_inc(key, increment = 1) return if key.nil? || key == "" (@rocket_job_perform_statistics ||= []) << (key.is_a?(Hash) ? key : [key, increment]) end |